/*******************************************************************************
 * Copyright (c) 2014 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of FastaManager.
 *
 *     LibProteomicSvg is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     LibProteomicSvg is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with LibProteomicSvg.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Anne Frambourg <aaubert@jouy.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.FastaManager;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import fr.inra.pappso.FastaManager.FastaFile.FastaFile;
import fr.inra.pappso.FastaManager.FastaFile.FastaFileFactory;
import fr.inra.pappso.FastaManager.exception.FastaException;
import fr.inra.pappso.FastaManager.fasta_transforme.FastaTransformer;
import fr.inra.pappso.FastaManager.fasta_transforme.output.FastaOutputFile;
import fr.inra.pappso.FastaManager.sequence_transforme.ReplaceSequenceIds;

public class FastaConvert {
	private static final Logger logger = Logger
			.getLogger(database_manager.class);

	public static HashMap<String, String> readCorrespondanceFile(
			InputStream correspFile) throws FastaException {
		BufferedReader in = new BufferedReader(new InputStreamReader(
				correspFile));
		HashMap<String, String> hashIds = new HashMap<String, String>();
		try {
			while (in.ready()) {
				String[] strPeak = in.readLine().trim().split(" ");

				if (strPeak.length == 2) {
					hashIds.put(strPeak[0], strPeak[1]);
				}
			}

			in.close();
		} catch (IOException e) {
			String message = "ERROR reading correp file:\n" + e.getMessage();
			logger.error(message);
			throw new FastaException(message, e);
		}
		return hashIds;
	}

	public static void main(String[] args) {
		PropertyConfigurator.configure(database_manager.class
				.getResource("/conf/log4j.properties"));
		logger.info("logged in");
		String correspondanceFilename = args[1];
		String inputFilename = args[2];
		String outputFilename = args[3];

		logger.debug("correspondanceFilename " + correspondanceFilename);
		logger.debug("inputFilename " + inputFilename);
		logger.debug("outputFilename " + outputFilename);

		FastaFile inFile = FastaFileFactory.newFastaFile(inputFilename);
		FastaOutputFile outputFile = null;
		try {
			outputFile = new FastaOutputFile(outputFilename);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(1);
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(1);
		} catch (FastaException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(1);
		}
		FastaTransformer transformer = null;
		try {
			transformer = new FastaTransformer(outputFile);
		} catch (FastaException e2) {
			logger.error(e2.getMessage());
			System.err.println(e2.getMessage());
			System.exit(1);
		}

		HashMap<String, String> correspondance = null;
		try {
			correspondance = readCorrespondanceFile(new FileInputStream(
					correspondanceFilename));
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			System.exit(1);
		} catch (FastaException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			System.exit(1);
		}

		try {
			transformer.addFastaFile(inFile);
		} catch (FastaException e1) {
			logger.error(e1.getMessage());
			System.err.println(e1.getMessage());
			System.exit(1);
		}
		try {
			transformer.transfert_sequences(new ReplaceSequenceIds(
					correspondance));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(1);
		}
	}
}
