package fr.inra.pappso.FastaManager.FastaFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.log4j.Logger;

import fr.inra.pappso.FastaManager.exception.FastaException;

public class FastaLocalFile implements FastaFile {
	private static final Logger logger = Logger.getLogger(FastaLocalFile.class);

	private File file;

	public FastaLocalFile(File file) {
		this.file = file;
	}

	@Override
	public InputStream getInputStream() throws IOException, FastaException {
		if (!this.exists()) {
			String message = "input Fasta file " + file + " does not exists";
			logger.error(message);
			throw new FastaException(message);
		}
		return new FileInputStream(file);
	}

	@Override
	public String getName() {
		return file.getName();
	}

	@Override
	public boolean delete() {
		return file.delete();
	}

	@Override
	public boolean exists() {
		return file.exists();
	}

	@Override
	public boolean renameTo(String newName) throws FastaException {
		if (!this.exists()) {
			String message = "source Fasta file " + file.getAbsolutePath()
					+ " does not exists";
			logger.error(message);
			throw new FastaException(message);
		}
		File fileName = new File(file.getParentFile()
				+ System.getProperty("file.separator") + newName);
		if (file.getAbsoluteFile().equals(fileName.getAbsolutePath())) {
			logger.debug("same name no need to move " + file.getAbsoluteFile());
			return true;
		}

		if (fileName.exists()) {
			String message = "moving FastaFile from " + file.getAbsolutePath()
					+ " to " + fileName.getAbsolutePath()
					+ " failed : destination already exists";
			logger.error(message);
			throw new FastaException(message);
		}
		return file.renameTo(fileName);
	}

}
