package fr.inra.pappso.FastaManager.FastaFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.log4j.Logger;

import com.googlecode.sardine.Sardine;
import com.googlecode.sardine.SardineFactory;

import fr.inra.pappso.FastaManager.exception.FastaException;

public class FastaWebdavFile implements FastaFile {
	private static final Logger logger = Logger
			.getLogger(FastaWebdavFile.class);

	private URL fastaUrl;
	private Sardine sardine;

	public FastaWebdavFile(Sardine sardine, URL url) {
		this.sardine = sardine;
		this.fastaUrl = url;
	}

	public FastaWebdavFile(URL fastaUrl) {
		String userInfo = fastaUrl.getUserInfo();
		sardine = SardineFactory.begin();
		if (userInfo != null) {
			String[] credential = userInfo.split(":");
			if (credential.length == 2) {
				sardine = SardineFactory.begin(credential[0], credential[1]);
			}
		}
		this.fastaUrl = fastaUrl;
	}

	@Override
	public InputStream getInputStream() throws IOException, FastaException {
		if (!this.exists()) {
			String message = "input Fasta file " + this.fastaUrl
					+ " does not exists";
			logger.error(message);
			throw new FastaException(message);
		}
		return sardine.get(fastaUrl.toString());
	}

	@Override
	public String getName() {
		return getUrlName();
	}

	@Override
	public boolean delete() {
		try {
			sardine.delete(this.fastaUrl.toString());
			return true;
		} catch (IOException e) {
			logger.error(e.getMessage());
			e.printStackTrace();
			return false;
		}
	}

	@Override
	public boolean exists() {
		try {
			return sardine.exists(this.fastaUrl.toString());
		} catch (IOException e) {
			logger.error(e.getMessage());
			e.printStackTrace();
			return false;
		}
	}

	@Override
	public boolean renameTo(String newName) throws FastaException {
		String source = this.fastaUrl.toString();
		logger.debug("move " + source);
		String destination = null;
		try {
			destination = getParentUrl().toString() + "/" + newName;
			logger.debug("to " + destination);
			if (!this.exists()) {
				String message = "moving FastaFile from " + source + " to "
						+ destination + " failed\n" + "source Fasta file "
						+ source + " does not exists";
				logger.error(message);
				throw new FastaException(message);
			}
			if (source.equals(destination)) {
				logger.debug("same name no need to move " + source);
				return true;
			}

			if (sardine.exists(destination)) {
				String message = "moving FastaFile from " + source + " to "
						+ destination + " failed : destination already exists";
				logger.error(message);
				throw new FastaException(message);
			}
			sardine.move(source, destination);
			return true;
		} catch (IOException e) {
			String message = "moving FastaFile from " + source + " to "
					+ destination + " failed";
			logger.error(message + "\n" + e.getMessage());
			e.printStackTrace();
			throw new FastaException(message);
		}
	}

	private String getUrlName() {
		File tmpFile = new File(this.fastaUrl.getPath());
		return tmpFile.getName();
	}

	private String getUrlParentPath() {
		File tmpFile = new File(this.fastaUrl.getPath());
		return tmpFile.getParent();
	}

	private URL getParentUrl() throws MalformedURLException {
		String newUrlStr = "";
		if (this.fastaUrl.getProtocol() != null) {
			newUrlStr += fastaUrl.getProtocol() + "://";
		}
		if (this.fastaUrl.getUserInfo() != null) {
			newUrlStr += fastaUrl.getUserInfo() + "@";
		}
		if (this.fastaUrl.getHost() != null) {
			newUrlStr += fastaUrl.getHost();
		}
		if (this.fastaUrl.getPort() > 0) {
			newUrlStr += ":"+this.fastaUrl.getPort();
		}
		if (getUrlParentPath() != null) {
			newUrlStr += getUrlParentPath();
		} else {
			newUrlStr += "/";
		}
		logger.debug("parent URL" + newUrlStr);
		return new URL(newUrlStr);
	}

}
