/*******************************************************************************
 * Copyright (c) 2014 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of FastaManager.
 *
 *     LibProteomicSvg is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     LibProteomicSvg is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with LibProteomicSvg.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Anne Frambourg <aaubert@jouy.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.FastaManager;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import fr.inra.pappso.FastaManager.FastaFile.FastaFile;
import fr.inra.pappso.FastaManager.FastaFile.FastaFileFactory;
import fr.inra.pappso.FastaManager.exception.FastaException;
import fr.inra.pappso.FastaManager.fasta_transforme.FastaTransformer;
import fr.inra.pappso.FastaManager.fasta_transforme.output.FastaOutputFile;
import fr.inra.pappso.FastaManager.sequence_transforme.ReplaceSequenceIds;

public class MergePgrouperFasta {
	private static final Logger logger = Logger
			.getLogger(database_manager.class);

	public static HashMap<String, String> readCorrespondanceFile(
			InputStream correspFile) throws FastaException {
		// >first line contains accession
		// second while description
		// third group number
		BufferedReader in = new BufferedReader(new InputStreamReader(
				correspFile));
		HashMap<String, String> hashIds = new HashMap<String, String>();
		int lineNumber = 0;
		try {
			while (in.ready()) {
				lineNumber++;
				String newCorresp = in.readLine();
				if (!newCorresp.isEmpty()) {
					String[] strPeak = newCorresp.trim().split(" ");
					if (!strPeak[0].startsWith(">")) {
						throw new FastaException(
								"no > sign as expected at line " + lineNumber);
					}
					String accession = strPeak[0].substring(1);
					if (accession.isEmpty()) {
						throw new FastaException("accession is empty at line "
								+ lineNumber);
					}
					// accession = accession.substring(0,
					// accession.lastIndexOf("|"));
					lineNumber++;
					String description = in.readLine().trim();

					lineNumber++;
					String groupNumber = in.readLine().trim();
					if (groupNumber.isEmpty()) {
						throw new FastaException(
								"groupNumber is empty at line " + lineNumber);
					}

					// logger.debug(accession + " " + groupNumber + "|"
					// + description);
					hashIds.put(accession, accession + "|" + groupNumber);
				}

			}

			in.close();
		} catch (IOException e) {
			String message = "ERROR reading corresp file:\n" + e.getMessage();
			logger.error(message);
			throw new FastaException(message, e);
		}
		return hashIds;
	}

	public static void main(String[] args) {
		PropertyConfigurator.configure(database_manager.class
				.getResource("/conf/log4j.properties"));
		logger.info("logged in");
		String correspondanceFilename = args[0];
		logger.debug("correspondanceFilename " + correspondanceFilename);
		String inputFilename = args[1];
		logger.debug("inputFilename " + inputFilename);
		String outputFilename = args[2];

		logger.debug("outputFilename " + outputFilename);

		FastaFile inFile = FastaFileFactory.newFastaFile(inputFilename);
		FastaOutputFile outputFile = null;
		try {
			outputFile = new FastaOutputFile(outputFilename);
		} catch (FileNotFoundException e) {
			logger.error(e.getMessage());
			e.printStackTrace();
			System.exit(1);
		} catch (UnsupportedEncodingException e) {
			logger.error(e.getMessage());
			e.printStackTrace();
			System.exit(1);
		} catch (FastaException e) {
			logger.error(e.getMessage());
			e.printStackTrace();
			System.exit(1);
		}
		logger.debug("open in writing mode fasta output file");
		FastaTransformer transformer = null;
		try {
			transformer = new FastaTransformer(outputFile);
		} catch (FastaException e2) {
			logger.error(e2.getMessage());
			System.err.println(e2.getMessage());
			System.exit(1);
		}

		HashMap<String, String> correspondance = null;
		try {
			logger.debug("reading correspondance file");
			correspondance = readCorrespondanceFile(new FileInputStream(
					correspondanceFilename));
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			System.exit(1);
		} catch (FastaException e1) {
			logger.error(e1.getMessage());
			e1.printStackTrace();
			System.exit(1);
		}

		logger.debug("read fasta input file");
		try {
			transformer.addFastaFile(inFile);
		} catch (FastaException e1) {
			logger.error(e1.getMessage());
			System.err.println(e1.getMessage());
			System.exit(1);
		}
		try {
			logger.debug("begin accession replacement");
			transformer.transfert_sequences(new ReplaceSequenceIds(
					correspondance));
		} catch (Exception e) {
			logger.error(e.getMessage());
			e.printStackTrace();
			System.exit(1);
		}
		System.exit(0);
	}
}
