package fr.inra.pappso.FastaManager.database;

import java.io.File;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import fr.inra.pappso.FastaManager.exception.FastaException;

public class RepositorysLocation {

	public static String LocalRepository = "LocalRepository";

	public static String WebdavRepository = "WebdavRepository";

	private static final Logger logger = Logger
			.getLogger(RepositorysLocation.class);

	private Hashtable<String, Element> repositories;

	static private File file;

	private Document doc;

	private Element racine;

	static {
		if (System.getProperty("os.name").contains("Linux")) {
			file = new File(System.getProperty("user.home")
					+ System.getProperty("file.separator")
					+ ".config/fastamanager/" + "database_manager.conf");
		} else {
			file = new File(System.getProperty("user.home")
					+ System.getProperty("file.separator")
					+ "database_manager.conf");
		}
		if (!file.getParentFile().exists()) {
			file.getParentFile().mkdirs();
		}
	}

	public RepositorysLocation() throws Exception {

		this.repositories = new Hashtable<String, Element>();

		try {
			// creation et ouverture d'un DOM
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder parser = factory.newDocumentBuilder();

			// on charge normalement le nouveau fichier conf ou on le crée
			if (!RepositorysLocation.file.exists()) {
				this.doc = parser.newDocument();
				this.racine = doc.createElement("database_manager");
				doc.appendChild(this.racine);
				write();
			} else {
				this.doc = parser.parse(RepositorysLocation.file);
				this.racine = this.doc.getDocumentElement();
			}

			// on charge les repository
			initialise();

			// on vérifie si on viens de l'ancienne version
			File old = new File(System.getProperty("user.home")
					+ System.getProperty("file.separator")
					+ "database_manager.xml");

			if (old.exists()) {
				logger.info("Import old repository from version 1.2");
				Document oldDoc = parser.parse(old);
				Element oldRacine = oldDoc.getDocumentElement();
				NodeList nl2 = oldRacine.getElementsByTagName("repository");
				Element repositorie = (Element) nl2.item(0);
				String oldLocation = repositorie.getAttribute("value");
				File oldLocationdirectory = new File(oldLocation);
				if (oldLocationdirectory.exists()
						&& oldLocationdirectory.isDirectory()) {
					File oldLocationFile = new File(oldLocation
							+ System.getProperty("file.separator")
							+ "database_manager.xml");
					logger.info("Move database_manager.xml to " + oldLocation);
					old.renameTo(oldLocationFile);
					this.addRepositories(oldLocation, LocalRepository,
							"Import from 1.2");
					this.write();
				}
			}

		} catch (Exception e) {
			throw e;
		}
	}

	private void initialise() {
		// Remplissage des index id de repositories
		NodeList nl = this.racine.getElementsByTagName("repository");
		repositories.clear();
		for (int i = 0; i < nl.getLength(); ++i) {
			Element database = (Element) nl.item(i);
			String id = database.getAttribute("id");
			repositories.put(id, database);
		}
	}

	public void write() throws Exception {
		// écriture d'un document
		try {
			TransformerFactory tfact = TransformerFactory.newInstance();
			Transformer transformer = tfact.newTransformer();
			transformer.setOutputProperty("encoding", "UTF-8");
			DOMSource source = new DOMSource(this.doc);
			PrintWriter fw = new PrintWriter(RepositorysLocation.file, "UTF-8");
			StreamResult result = new StreamResult(fw);
			transformer.transform(source, result);
		} catch (Exception e) {
			throw new FastaException("Problems during saved of the file : "
					+ RepositorysLocation.file.getName());
		}
	}

	public String selectNewid() {
		String id = "rp";
		boolean id_ok = false;
		int i = 0;
		do {
			i++;
			id_ok = true;
			Enumeration<String> E = this.repositories.keys();
			while (E.hasMoreElements()) {
				String cle = E.nextElement();
				Element database = (Element) this.repositories.get(cle);
				if (database.getAttribute("id").equals(id + i))
					id_ok = false;
			}

		} while (!id_ok);
		id = id + i;
		return (id);
	}

	public void deleteRepository(Element repository) {
		racine.removeChild(repository);
		initialise();
	}

	public Element getRepositorieElement(String signet) {
		Enumeration<Element> E = this.repositories.elements();
		while (E.hasMoreElements()) {
			Element element = E.nextElement();
			if (element.getAttribute("signet").equals(signet))
				return (element);
		}
		return (null);
	}

	public Repository getRepositorie(Element element) throws Exception {
		Repository repo = null;
		if (this.getRepositoryType(element).equals(
				RepositorysLocation.LocalRepository)) {
			repo = new Repository(element);
		} else if (this.getRepositoryType(element).equals(
				RepositorysLocation.WebdavRepository)) {
			repo = new RepositoryWebdav(element);
		}
		return (repo);
	}

	public Element addRepositories(String resources, String type, String signet)
			throws Exception {
		// Vérifie si il n'existe pas et on renvoie le bon element
		Enumeration<Element> E = this.repositories.elements();
		while (E.hasMoreElements()) {
			Element element = E.nextElement();
			if (element.getAttribute("resource").equals(resources)) {
				return (element);
			}
			if (element.getAttribute("signet").equals(signet)) {
				return (element);
			}
		}

		if (!type.equals(RepositorysLocation.LocalRepository)
				&& !type.equals(RepositorysLocation.WebdavRepository))
			throw new FastaException(
					"This type of repository are not supported " + type);
		String id = selectNewid();
		Element repo = doc.createElement("repository");
		repo.setAttribute("id", id);
		repo.setAttribute("resource", resources);
		repo.setAttribute("type", type);
		repo.setAttribute("signet", signet);
		if (type.equals(RepositorysLocation.WebdavRepository)) {
			repo.setAttribute("login", "");
			repo.setAttribute("password", "");
		}
		this.racine.appendChild(repo);
		this.initialise();

		return (repo);
	}

	public Hashtable<String, Element> getRepositories() {
		return (this.repositories);
	}

	public String getRepositoryResource(Element element) {
		String species = element.getAttribute("resource");
		return (species);
	}

	public String getRepositoryType(Element element) {
		String species = element.getAttribute("type");
		return (species);
	}

	public String getRepositorySignet(Element element) {
		String species = element.getAttribute("signet");
		return (species);
	}

	public String getRepositoryLogin(Element element) {
		String species = element.getAttribute("login");
		return (species);
	}

	public String getRepositoryPassword(Element element) {
		String species = element.getAttribute("password");
		return (species);
	}

	public void setRepositoryResource(Element element, String resource) {
		resource = resource.replaceFirst("/$", "");
		element.setAttribute("resource", resource);
	}

	public void setRepositoryLogin(Element element, String login) {
		element.setAttribute("login", login);
	}

	public void setRepositoryPassword(Element element, String password) {
		element.setAttribute("password", password);
	}

	public void setRepositoryType(Element element, String type)
			throws Exception {
		if (!type.equals(RepositorysLocation.LocalRepository)
				&& !type.equals(RepositorysLocation.LocalRepository))
			throw new FastaException(
					"This type of repository are not supported " + type);
		element.setAttribute("type", type);
	}

	public void setRepositorySignet(Element element, String signet) {
		element.setAttribute("signet", signet);
	}

	public String getInformation(Element repo) {
		return (this.getRepositorySignet(repo) + " : " + this
				.getRepositoryResource(repo));
	}

}
