/*******************************************************************************
 * Copyright (c) 2014 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of FastaManager.
 *
 *     LibProteomicSvg is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     LibProteomicSvg is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with LibProteomicSvg.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Anne Frambourg <aaubert@jouy.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/
package fr.inra.pappso.FastaManager.sequence_transforme;

import java.util.HashMap;

import org.apache.log4j.Logger;

import fr.inra.pappso.FastaManager.sequence.sequence;
import fr.inra.pappso.FastaManager.sequence.sequence_prot;

/** this will replace accessions in a fasta file, leaving the description unchanged
 * it needs a correspondance table that associates the old accession number to the new one
 *
 * @author langella
 *
 */
public class ReplaceSequenceIds extends seq_transforme {
	private static final Logger logger = Logger
			.getLogger(ReplaceSequenceIds.class);
	private HashMap<String, String> old2newId = new HashMap<String, String>();

	public ReplaceSequenceIds(HashMap<String, String> old2newId) {
		this.seq_out = new sequence_prot[1];
		this.old2newId = old2newId;
	}

	public sequence[] transforme(sequence sequenceIn) throws Exception {
		String newAccession = this.old2newId.get(sequenceIn.get_accession());
		this.seq_out[0] = new sequence_prot();
		if (newAccession != null) {
			logger.debug(newAccession + " found");
			this.seq_out[0].set_accession(newAccession);
			this.seq_out[0].set_sequence(sequenceIn.get_sequence());
			this.seq_out[0].set_description(sequenceIn.get_description());
		} else {
			//logger.debug(sequenceIn.get_accession() + " not found");
			this.seq_out[0].set_accession(null);
			this.seq_out[0].set_sequence(null);
			this.seq_out[0].set_description(null);
		}

		return (this.seq_out);
	}

	public String get_type() {
		return ("Replace sequence IDs");
	}

	public String get_sequence_type() {
		return ("PROT");
	}

}
